@@ -1,5 +1,5 @@
 /*=============================================================================
-  Copyright (C) 2003-2019 Fisheries and Oceans Canada
+  Copyright (C) 2003-2022 Fisheries and Oceans Canada
 
   This file is part of PBS Mapping.
 
@@ -39,8 +39,8 @@
 #include <stdio.h>
 #include <math.h>
 #define Rprintf printf
-typedef int Sint;
-typedef double Sfloat;
+/* typedef int Sint;
+typedef double Sfloat; */
 
 #else /* not defined STANDALONE */
 
